/**
 * RemoteKVM - system.js
 * version 0.9
 * @author firejune(to@firejune.com)
 * @author Jeong Huiju(hjjeong@rsupport.com)
 * @copyright © 2012 RSUPPORT. CO, LTD. All rights Reserved.
 */
(function($, win, doc){

  $.extend(jQuery.easing,{
    def: 'customEasing',
    customEasing: function(x,t,b,c,d){ if((t/=d/2) < 1) return c/2*t*t*t*t*t + b; return c/2*((t-=2)*t*t*t*t + 2) + b; }
  });
  var $center
    , $right
    , $current
    , $anchor
    , $devices = []
    , $first = true
    , $win = $(win)
    , $doc = $(doc)
    , $body = $(doc.body)
    , $content = $('#content')
    , $message = $('#message')
    , $controllers = /(screens|records|logs|devices|users|settings|downloads|account|about)/
    , $rules = {
      // users
      id: {required: true, letters: true, rangelength: [1, 32]},
      pw: {newUserRequired: true, rangelength: [4, 32]},
      pw2: {newUserRequired: true, rangelength: [4, 32], equalTo: 'input[name="pw"]'},
      // devices
      name: {required: true, letters: true, rangelength: [1, 32]},
      ip4: {required: true, ipv4: true},
      subnet4: {required: true, netmask: true},
      gateway4: {ipv4: true},
      web_port: {required: true, digits: true},
      dns1: {ipv4: true},
      dns2: {ipv4: true},
      //settings
      'pc_mac':{mac: true},
      'ip[]': {required: true, ipv4: true},
      'netmask[]': {required: true, netmask: true}
    };

  /* additional validate rules */
  $.validator.addMethod("newUserRequired", function(value, element, param) {
    var newUser = $anchor.attr('href').match('#new');
    return !newUser || newUser && value;
  }, $.kvm.string.VALIDATE_001); //STR: This field is required.

  $.validator.addMethod("letters", function(value, element) {
    return this.optional(element) || /^[a-z]{1}[\w-]+$/i.test(value);
  }, $.kvm.string.VALIDATE_018); //STR: Please enter a valid name

  $.validator.addMethod("netmask", function(value, element, param) {
    var valid = true
      , correctRange = '128 192 224 240 248 252 254 255 0'.split(' ')
      , m = value.split('.');

    for (var i = 0; i <= 3; i ++) {
      if (correctRange.indexOf(m[i]) == -1) {
        valid = false;
        break;
      }
    }
    if (m[0] == 0 || (m[0] != 255 && m[1] != 0) || (m[1] != 255 && m[2] != 0) || (m[2] != 255 && m[3] != 0)) valid = false;

    return this.optional(element) || /^\d+$/ && value >= 0 && value <= 32 || valid;
  }, $.kvm.string.VALIDATE_019); //STR: Please enter a valid Mask.

  $.validator.addMethod("ipv4", function(value, element, param) {
    return this.optional(element)
      || /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/i.test(value);
  }, $.kvm.string.VALIDATE_020); //STR: Please enter a valid IP v4 address.

  $.validator.addMethod("ipv6", function(value, element, param) {
    return this.optional(element)
      || /^((([0-9A-Fa-f]{1,4}:){7}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}:[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){5}:([0-9A-Fa-f]{1,4}:)?[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){4}:([0-9A-Fa-f]{1,4}:){0,2}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){3}:([0-9A-Fa-f]{1,4}:){0,3}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){2}:([0-9A-Fa-f]{1,4}:){0,4}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(([0-9A-Fa-f]{1,4}:){0,5}:((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(::([0-9A-Fa-f]{1,4}:){0,5}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|([0-9A-Fa-f]{1,4}::([0-9A-Fa-f]{1,4}:){0,5}[0-9A-Fa-f]{1,4})|(::([0-9A-Fa-f]{1,4}:){0,6}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,7}:))$/i.test(value);
  }, $.kvm.string.VALIDATE_021); //STR: Please enter a valid IP v6 address.

  $.validator.addMethod("mac", function(value, element, param) {
    return this.optional(element)
      || /^[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]$/i.test(value);

  }, $.kvm.string.VALIDATE_023); //STR: Please enter a valid IP v6 address.

  /* setup for ajax request */
  $.ajaxSetup({
    async: true,
    type: 'get',
    dataType: 'json',
    timeout: 15000,
    error: function(xhr) {
      //if (xhr.status == 401 || xhr.status == 403) location.href = '/signin/?return=' + location.hash.replace('#', '');
      if (xhr.status == 401 || xhr.status == 403){
        alert($.kvm.string.ALERT_002);
        location.href = '/signout';
      }
      else {
        var data = $.parseJSON(xhr.responseText);
        if (data && typeof data == 'object') {
          if (data.fieldName == 'ignore') return;
          if (data.fieldName) $('form').find('*[name="' + data.fieldName + '"]').css('border-color', 'red').focus();
          else status(data.statusText, data.statusType);
        }// else !xhr.url.match('devices/reboot') && status(xhr.status + ' - ' + xhr.statusText, 'error')
      }

      if (xhr.statusText == 'abort' && this.jsonpCallback) {
        win[this.jsonpCallback] = function() {
          //console.warn('aborted data', xhr);
          delete win[this.jsonpCallback];
        }
      }

      if (data && data.fieldName == "force" && this.url.match('/devices/delete')) {
        var data = toQueryParams(this.data);
        !data.force && dialog([
          '<h3 class="warn">'+$.kvm.string.DIALOG_003+'</h3>', //STR: Does not respond. Delete data only?
          '<p>'+$.kvm.string.DIALOG_004+'</p>' //STR: Can not synchronize the device does not respond, so you want to delete data only.
        ].join('\n'), {
          callback: function(button) {
            button.form.method = "post";
            button.form.action = "/devices/delete.php";
            $(button.form).append(template([
              '<input name="force" value="Y" type="hidden"/>',
              '<input name="seq" value="{value}" type="hidden"/>'
            ].join('\n'), {
              value: data.seq
            }));
            submit(button.form);
          }
        });
      }

      // handle for connection error
      if (this.dataType == 'jsonp' && xhr.statusText != 'abort') {
        this.success && this.success([]);
        var url = this.url.split(/\/records\/|\/logs\//)[0];
        var controller = getController();
        url && request.errors.indexOf(url) == -1 && request.errors.push(url);
        status($.kvm.string.STATUS_012A + capitalize(controller) + $.kvm.string.STATUS_012B, 'error'); //STR: Failed reading ... data.
        //console.log(controller);
        if(controller.match(/logs|records/)){
          $('#list-table').find('tbody').html('<tr><td class="error-cell" colspan="'+((controller == 'logs') ? '5' : '4')+'">'+$.kvm.string.STATUS_012C+'</td></tr>');
        }
      }

      $message.removeClass('loading');
      rightOverlay.close();
      overlay.close();
    },

    beforeSend: function() {
      if (request.ajax && request.ajax.readyState != 4) request.ajax.abort();
      if (this.dataType == 'jsonp') //console.info(this.url);
      /*
      for (var i = 0; i < request.errors.length; i++) {
        if (this.url.match(request.errors[i])) return true;
      }
      */
      clearTimeout(request.timeout);
      if (status.refresh) status();
      $message.addClass('loading');
      rightOverlay(true);
      if($('#record-detail').length > 0) $('#record-detail').appendTo("#content-body").hide().find(".snapshots img").remove();
    },

    complete: function() {
      $('#items a').each(function() {
        for (var i = 0; i < request.errors.length; i++) {
          if (this.href.match(request.errors[i])) return $(this).parent('li').addClass('error');
        }
      });

      request.timeout = setTimeout(function() {
        if ($first) status();
        $message.removeClass('loading');
        rightOverlay.close();
        $first = false;
      }, 256);
    }
  });

  /** *************************************
   * key event fix for mozilla browsers
   * @param {String} key event type
   * @return {Object} jquery object
   *************************************** */
  $.fn.keyEventFix = function(eventType) {
    if (!eventType) eventType = 'keypress';

    function watchInput(input) {
      var $input = $(input);
      var value = $.trim($input.val());

      // trigger event
      if (value && input.db != value) $input.trigger(eventType);
      input.db = value;

      if (input.keyEventCheck) win.clearInterval(input.keyEventCheck);
      input.keyEventCheck = win.setInterval(function() {
        watchInput(input);
      }, 100);
    }

    function focusFunc(event) {
      if (!this.keyEventCheck) watchInput(this);
    }

    function blurFunc(event) {
      if (this.keyEventCheck) {
        win.clearInterval(this.keyEventCheck);
        this.keyEventCheck = null;
        this.db = null;
      }
    }

    return $.browser.mozilla == false ? this : this.each(function() {
      this.keyEventCheck = null;
      this.db = null;

      $(this).focus(focusFunc).blur(blurFunc);
    });
  };

  /**
   * spotlight plugin
   * @param {Object} parent element
   * @return {Object} jquery object

  $.fn.spotlight = function(parent) {
    //console.log('spotlight', arguments);

    var overlays = []
      , offsetBottom = 0, parent = (parent || $right).addClass('spotlight')
      , offsetHeight = parent.prop('offsetHeight'), scrollHeight = parent.prop('scrollHeight')
      , offsetWidth = parent.prop('offsetWidth') + (offsetHeight == scrollHeight ? 1 : -17);

    this.each(function(i) {
      this.spotlighted = true;

      var target = $(this)
        , width = target.prop('offsetWidth'), height = target.prop('offsetHeight'), top = target.prop('offsetTop')
        , left = target.prop('offsetLeft'), right = left + width, bottom = top + height;

      // top
      overlays.push({
        top: 0 + offsetBottom, left: 0, width: offsetWidth, height: top - offsetBottom
      });

      // left
      overlays.push({
        top: top, left: 0, width: left, height: height
      });

      // right
      overlays.push({
        top: top, right: 0, width: (right - offsetWidth) * -1, height: height
      });

      // bottom
      i && overlays.push({
        top: bottom, left: 0, width: offsetWidth, height: scrollHeight - bottom
      });
      offsetBottom += bottom;
    });

    $.each(overlays, function() {
      var block = $('<div class="overlay"></div>').css('opacity', 0);
      block.css(this);
      parent.append(block.animate({opacity: 0.2}, 500));
    });
    return this;
  };
  */

  /** *******************************
   * typewriter plugin
   * @return {Object} jquery object
   ******************************** */
  $.fn.typewriter = function() {
    this.each(function() {
      var $ele = $(this), str = $ele.text(), progress = 0;
      $ele.text('');
      var timer = setInterval(function() {
        $ele.text(str.substring(0, progress++) + (progress & 1 ? '_' : ''));
        if (progress > str.length) clearInterval(timer);
      }, 100);
    });
    return this;
  };

  /* shared functions */

  /** *********************
   * empty function
   ********************** */
  function emptyfunc(){
    return false;
  }

  /** ********************************************
   * simple template engine
   * @param {String} template source
   * @param {Object} dictionary object like json
   * @return {String} replaced string
   ********************************************* */
  function template(src, dic) {
    //console.log('template', arguments);
    for (var key in dic) src = src.replace(new RegExp('{' + key + '}', 'g'), dic[key]);
    return src;
  }

  /** *************************************
   * capitalize string
   * @param {String} some word
   * @return {String} capitalized word
   *************************************** */
  function capitalize(str) {
    //console.log('capitalize', arguments);
    return str.charAt(0).toUpperCase() + str.substring(1).toLowerCase();
  }

  /** ************************************
   * jsonlize
   * @param {Object} serialized array
   * @return {Object} jsonlized object
   ************************************** */
  function jsonlize(array) {
    //console.log('jsonlize', arguments);

    var json = {};
    $.each(array, function () {
      if (json[this.name] && typeof json[this.name] != 'object') json[this.name] = [json[this.name], this.value];
      else if (json[this.name]) json[this.name].push(this.value);
      else json[this.name] = this.value;
    });

    return json;
  }

  /** *********************************
   * toQueryParams
   * @param {String} query string
   * @return {Object} object params
   ********************************** */
  function toQueryParams(query) {
    //console.log('toQueryParams', arguments);

    var params = {}, string = query, i = 0;
    do {
      params[string.split("=")[0].replace(/&/, "")] = ( string.split("=")[1] ) ? string.split("=")[1].split(/&|$/)[0] : '';
      string = ( string.split("=")[1] ) ? string.split(string.split("=")[1].split(/&|$/)[0])[1] : '';
      i++;
    } while(i < (query.split("=").length - 1));

    return params;
  }

  /** ***************************************
   * dialog
   * @param {String} contents html string
   * @param {Function} confirmed callback
   ***************************************** */
  dialog.stack = [];
  function dialog(html, options) {
    //console.info('dialog', arguments);
    dialog.close();
    options = $.extend({
      modal: true,
      callback: null,
      cancel_callback: null,
      labels: {
        ok: $.kvm.string.DIALOG_001A, //STR: OK
        cancel: $.kvm.string.DIALOG_001B //STR: Cancel
      }
    }, options);
    if (!dialog.wrapper || !dialog.wrapper.length) {
      $body.append([
        '<div id="dialog-wrapper" style="display: block;">',
          '<div id="dialog">',
            '<button id="button-close-preview">Close</button>',
            '<div class="dialog-body">',
              html,
            '</div>',
            '<form class="buttons" id="dialog-buttons" style="display: none;">',
              '<button class="button blue" id="confirm-yes" type="button">yes</button>',
              '<button class="button" id="confirm-no" type="button">no</button>',
            '</form>',
          '</div>',
        '</div>'
      ].join('\n'));

      dialog.wrapper = $('#dialog-wrapper');
      dialog.frame = $('#dialog');
      $('#button-close-preview').click(dialog.close);
    }
    else if (dialog.wrapper.is(':visible')) return dialog.stack.push(arguments);
    else dialog.wrapper.show().find('.dialog-body').html(html);

    var dialog_top = ($win.height() - dialog.frame.height()) / 2 - 50;
    dialog.frame.css('margin-top', dialog_top > 15 ? dialog_top : 15);

    $('#dialog-buttons')[options.callback ? 'show' : 'hide']().find('input').remove().removeAttr('action').removeAttr('method');

    if (options.callback) {
      $('#confirm-yes').text(options.labels.ok).unbind('click').click(function() {
        dialog.close();
        return options.callback(this) || !!this.form.action;
      });
    }
    if (options.labels.cancel == '') {
       $('#confirm-no').hide();
    }
    else {
      $('#confirm-no').text(options.labels.cancel).unbind('click').click(function(){
        dialog.close();
        return options.cancel_callback ? options.cancel_callback(this) : false;
      });
    }
    dialog.visible = true;
  }

  dialog.close = function() {
    console.log('dialog.close');
    dialog.wrapper && dialog.wrapper.hide();
    dialog.stack.length && dialog.apply({}, dialog.stack.pop());
    dialog.visible = false;

    $('#dialog').find('input[type=hidden]').remove().end()
      .find('.rview-dialog').appendTo($('#rview-dialog-forms'))
      .find(':text,:password').unbind('keypress').val('');

    return false;
  };

  function checkUpdate() {
    //console.info('checkUpdate', arguments);

    var firmware = $.kvm.firmware;
    if (!firmware || checkUpdate.confirmed) return status();

    /** update check temporarily disable
    firmware.current < firmware.latest ?
      //STR: New firmware( ... )is now available. You can download it now.
      status('<a href="#settings/firmware">' + $.kvm.string.STATUS_002A + firmware.latest +  $.kvm.string.STATUS_002B + '</a>', 'info') :
      //STR: You have firmware version ... There are no updates available.
      status($.kvm.string.STATUS_003A + firmware.current + $.kvm.string.STATUS_003B) ;
    */
  }

  /** *************************************
   * status
   * @param {String} status message text
   * @param {String} status message type
   ************************************** */
  function status(text, type) {
    //console.info('status', text);

    if (typeof text == 'number') return checkUpdate();
    if (!text) text = $.kvm.string.STATUS_001A + capitalize($.kvm.user.name) + $.kvm.string.STATUS_001B; //STR: Welcome back,
    if (!type) type = 'log';

    status.timeout && clearTimeout(status.timeout);
    if (status.text == text && status.type == type) return false;

    $message.stop().animate({'opacity': 0}, 256, function() {
      var currentWidth = $(this).removeClass('loading').width();
      $(this).toggleClass('warn', type == 'warn').toggleClass('error', type == 'error').toggleClass('info', type == 'info');
      var width = $(this).removeClass('loading').html(text).width('auto').width();
      $(this).width(currentWidth).animate({opacity: 1, width: width}, 512, function() {
        status.text = status.type = null;
      });
    });

    status.text = text;
    status.type = type;

    if (status.refresh = type.match(/error|warn|info/) || $first) status.timeout = setTimeout(status, 4000);
  }

  /** *****************************************
   * display error balloon
   *
   * @param {Object} data object like json
   ******************************************** */
  function error(text, input) {
    //console.log('error', arguments);

    var p = input.next('p.error');
    if (p.length) p.html(text);
    else input.after('<p class="error">' + text + '</p>');
  }

  /** ******************************
   * display overlay
   * @param {String} message text
   ******************************** */
  function overlay(message) {
    //console.log('overlay', arguments);

    if (!overlay.wrapper || !overlay.wrapper.length) {
      $body.append([
        '<div id="overlay">',
          '<p><span>' + message + '</span></p>',
        '</div>'
      ].join('\n'));

      overlay.wrapper = $('#overlay');
      overlay.body = overlay.wrapper.find('span');
    } else {
      overlay.body.html(message);
    }

    overlay.wrapper.css('opacity', 0).show().animate({opacity: 0.66}, 120);
    overlay.interval = setInterval(function() {
      overlay.body.typewriter();
    }, 5000);
    overlay.body.typewriter();
  }

  overlay.close = function() {
    //console.log('overlay.close');

    overlay.wrapper && overlay.wrapper.animate({opaicty: 0}, 120, function() {
      overlay.wrapper.hide();
      clearInterval(overlay.interval);
    });
    return false;
  };

  function rightOverlay(loading){
    if(rightOverlay.overlay) return false;
    else{
      $right && !$('#right-overlay').length && $right.append('<div id="right-overlay"></div>');
      rightOverlay.overlay = $('#right-overlay');
    }
    rightOverlay.overlay[loading ? 'addClass':'removeClass']('loading').show();
  }
  rightOverlay.close = function(){
    rightOverlay.overlay && rightOverlay.overlay.hide();
    rightOverlay.overlay = null;
  }

  /** ****************************************
   * data fill automation
   * @param {Object} data object like json
   ***************************************** */
  function fillData(data) {
    //console.info('fillData', arguments);
    for (var name in data) {
      var node = $('*[name=' + name + ']');
      node.length && node.val(data[name]);
    }
  }

  /** *****************
   * clear data
   ****************** */
  function emptyData() {
    //console.info('emptyData', arguments);

    var controller = getController();
    $content.animate({scrollTop: 0}, 400);
    $form.find('p.error').remove();
    $('form input[name]').val('');

    if (controller == 'users') {
      $('input[name="id"]').val(getNewLabel()).focus();
      $('.required').show();
      $('#assigned').hide().find('.item').remove();
      $('#account-type').html([
        '<option value="admin" selected="selected">'+ $.kvm.string.STR_ADMIN +'</option>', //STR: Administrator
        '<option value="user">'+ $.kvm.string.STR_USER +'</option>' //STR: User
      ].join('\n'));
    } else {
      $form.find('.error').removeClass('error').filter('p').remove();
      $('input[disabled]').removeAttr('disabled');
      $('input[name="name"]').val(getNewLabel());
      $('input[type="text"],input[type="password"]').attr("readonly","readonly").addClass("readonly");
      $('input[name="ip4"],input[name="web_port"]').removeAttr("readonly").removeClass("readonly").each(function(){this.spotlighted = true})
        .unbind('keypress').keyEventFix().keypress(searchDevice).first().focus();
    }
  }

  /** ******************************************
   * get the controller name
   * @return {String} current controller name
   ******************************************** */
  function getController(href) {

    var controller = (href || location.hash).match($controllers);
    //console.log('getController',controller);
    controller = controller ? controller[0] : 'screens';
    var user_type = $.kvm.user.type;
    if(user_type == 'admin' || user_type == 'user') {
      if(controller == 'settings'){
        if(href.match('downloads.php')) controller = "downloads";
        else if(href.match('about.php')) controller = "about";
      }
      if(user_type == 'user' && controller == 'users') controller = 'account';
    }
    return controller;
  }

  /** *********************************************
   * get a new item label from controller model
   * @return {String} item label
   ********************************************** */
  function getNewLabel() {
    //console.log('getNewLabel');

    var label = getController();
    label = label.substr(0, label.length - 1);
    if (label == 'device')  label = 'NEW-' + label.toUpperCase();
    else  label = 'new-' + label;

    return label;
  }

  /** ******************************
   * get properties
   * @param {Object} element
   * @return {Object} properties
   ******************************* */
  function getProperties(data) {
    //console.log('getProperties');
    data = (moreAction.anchor != null) ? moreAction.anchor.parent() : data.parents("li");
    var connect_domain = location.href.split("//")[1].split("/")[0].split(':');
    return {
      name: data.attr("data-name") || data.parents("ul").attr("data-name"),
      seq: data.attr("id") || data.parents("ul").attr("id"),
      ip: data.attr("data-ip") || data.parents("ul").attr("data-ip") || connect_domain[0],
      port: data.attr("data-port") || data.parents("ul").attr("data-port") || connect_domain[1] || '80'
    };
  }

  /** ***************************
   * screenshot action
   ****************************** */
  function screenAction(){
    if ($(this).hasClass('normal')) {
      remoteAccess($(this));
      //console.log("screenAction");
    }
    return false;
  }

  /* contextmenu controllers */
  /** ***************************
   * open action menu
   * @return {Boolean} false
   ****************************** */
  function moreAction() {
    //console.log('moreAction', this);
    var anchor = $(this)
      , menuHeight
      , offset = anchor.offset()
      , scrollHeight = $win.height()
      , scrollTop = 0
      , wrapperLeft = $right.offset().left
      , controller = getController()
      , top = offset.top + scrollTop + 24;
    if (moreAction.menu && moreAction.menu.is(':visible')) return moreAction.close();
    //if (!moreAction.menu || !$('#contextmenu').length || moreAction.controller != controller) {
      moreAction.menu = moreAction.create();
      moreAction.controller = controller;
    //}
    moreAction.anchor = anchor;

    var this_parent = anchor.parent().attr("class"),
        this_state = anchor.prev().attr("class");

    if(this_parent.match('local')) $('#maBtn_open').remove();
    if(this_state.match('error')) $('#maBtn_remote').remove();
    if(this_parent.match(/pending|new/)) $('#maBtn_setting, #maBtn_reboot').remove();
    if(this_state.match(/notconnect|loading/)) {
      $('#contextmenu>li').remove();
      moreAction.menu.css("height",0);
    }

    moreAction.menu.css({
      top: scrollHeight < top + (menuHeight = moreAction.menu.height()) ? top - menuHeight - 23 : top + 1,
      left: offset.left - wrapperLeft -2,
      width: anchor.prop('offsetWidth') +2
    }).show(0);
    return false;
  }

  /** *******************************
   * create context menu items
   * @return {Object} jquery object
   ******************************** */
  moreAction.create = function() {
    //console.log('moreAction.create');

    var items = []
      , menus = $('<ul class="contextmenu" id="contextmenu"></ul>');

    if (getController() == 'screens') {
      items = [
        {name: $.kvm.string.MOREACTION_001, action: remoteAccess, id: "remote"},  //STR: REMOTE
        {name: $.kvm.string.MOREACTION_002, action: viewRecords, id: "record"},   //STR: View Records
        {name: $.kvm.string.MOREACTION_003, action: viewLogs, id: "log"} //STR: View logs
      ];

      if($.kvm.user.type != 'user'){
        items.push(
          {name: $.kvm.string.MOREACTION_004, action: deviceSetting, id: "setting"},//STR: Device Setting
          {name: $.kvm.string.MOREACTION_005, action: rebootDevice, id: "reboot"}, //STR: Reboot Device
          {name: $.kvm.string.MOREACTION_007, action: wakepc, id: "wakepc"}, //STR: Wake PC
          {name: $.kvm.string.MOREACTION_006, action: openWebAdmin, id: "open"}  //STR: Open Web Admin Page
        );
      }
    }

    $.each(items, function(i, obj) {
      $('<a href="#">' + obj.name + '</a>').click(function() {
        obj.action();
        moreAction.close();
        return false;
      }).wrap('<li id="maBtn_'+obj.id+'"></li>').parent().appendTo(menus);
    });

    return menus.appendTo($right);
  };

  /** ***********************
   * close action menu
   ************************ */
  moreAction.close = function(){
    //console.log('moreAction.close');
    moreAction.menu && moreAction.menu.remove();
    moreAction.anchor = null;
  }

  /** ***********************
   * device setting
   ************************ */
  function goCommandPage(command,seq){
    //console.log(command+","+seq);
    var propertie = getProperties();
  }

  function deviceSetting() {
    //console.log('deviceSetting');

    var propertie = getProperties();
    //$('#left li.current').removeClass('current');
    //$('#left li[data-name="devices"]').addClass('current');
    //request('/devices/?seq=' + propertie.seq);
    location.href = '#devices/' + propertie.name;
  }

  /** ******************
   * view records
   ******************* */
  function viewRecords(){
    //console.log('viewRecords');
    var propertie = getProperties()
       ,goUrl = "#records";
    if(propertie.seq != $.kvm.device.seq) goUrl += '/' + propertie.name;
    location.href = goUrl;
  }

  /** ******************
   * view logs
   ******************* */
  function viewLogs(){
    //console.log('viewLogs');
    var propertie = getProperties()
       ,goUrl = "#logs";
    if(propertie.seq != $.kvm.device.seq) goUrl += '/' + propertie.name;
    location.href = goUrl;
  }

  /** *****************
   * download record
   ****************** */
  function downloadRecord(){
    //console.log('downloadRecord');
    var tr = $(this).parents('tr');
    var addr = $("#lastRow").attr('data-addr');
    location.href = 'http://' + addr + tr.attr('data-file-path') + tr.attr('data-file-name');
    setLog('download', $('#items li.current').attr('id'), tr.attr('data-file-name'));
  }

  function openWebAdmin(){
    //console.log('openWebAdmin');
    var propertie = getProperties($(this));
    window.open('http://' + propertie.ip + ":" + propertie.port + '/');
  }

  /** ******************
   * delete record
   ******************* */
  function deleteRecord(){
    //console.log('deleteRecord');
    var tr = $(this).parents('tr');
    var addr = $("#lastRow").attr('data-addr');

    dialog([
      '<h3 class="warn">'+ $.kvm.string.DIALOG_005 +'</h3>', //STR: Delete Record Item
      '<p>'+ $.kvm.string.DIALOG_006 +'</p>' //STR: Are you sure you want to continue?
    ].join('\n'), {
      callback: function() {
        $.ajax({
          url: 'http://' + addr + '/records/delete.php?file_name=' + tr.attr('data-file-name'),
          type: 'post',
          success: function() {
            $("#record-detail").appendTo("#content-body").hide().find(".snapshots img").remove();
            tr.prev().remove().end().remove();
          }
        });
      }
    });
  }
  /* plugin controller */
  /** *****************************************
   * create plugin
   * @return {Object} target element or null
   ****************************************** */
  function plugin() {
    //console.log('create plugin');

    var pluginPath = ""
      , pluginCLSID = ""
      , pluginMimeType = "application/x-remotekvmplugin"
      , div = doc.createElement("div");

    div.style.cssText = "visibility: hidden;position: absolute;width: 0px;height: 0px;";
    doc.body.appendChild(div);

    // supports NPAPI Browsers
    if ($.browser.mozilla || $.browser.webkit || $.browser.opera) {
      div.innerHTML = '<embed id="plug-in" pluginspage="' + pluginPath + '" type="' + pluginMimeType + '" style="margin:0; padding:0; width:100%; height:100%;"></embed>';
      return doc.getElementById('plug-in');
    } else if ($.browser.msie) {
      div.innerHTML = '<object id="plug-in" codebase="' + pluginPath + '" classid="' + pluginCLSID + '" type="' + pluginMimeType + '" style="margin:0; padding:0; width:100%; height:100%;"></object>';
      return doc.getElementById('plug-in');
    } else status($.kvm.string.STATUS_004, 'error'); //STR: Can not support plugin this browser.

    b.innerHTML = "<div></div>";
    return null;
  }

  /** ********************************
   * plugin valid
   * @return {Boolean} plugin valid
   ********************************** */
  plugin.valid = function(callback,args) {
    //console.log('plugin.valid', arguments);
    var _callback = callback ? eval(callback) : function(){};
    plugin.valid.ok = false;

    if (plugin.installed === undefined) plugin.installed = plugin();

    //get cookie
    var cookie = document.cookie,
        cookie_name = "plugin_confirm=",
        cookie_start = cookie.indexOf(cookie_name),
        cookie_value = "";

    if(cookie_start > -1){
      cookie_end = (cookie.indexOf(";", cookie_start) == -1) ? cookie.length : cookie.indexOf(";", cookie_start);
      cookie_value = cookie.substring(cookie_start + cookie_name.length, cookie_end);
    }

    if(plugin.installed && plugin.installed.valid){
      var installed_version = plugin.installed.version.split('.');
      var latest_version = $.kvm.plugin.latest.split('.');
      installed_version.pop();
      latest_version.pop();
      installed_version = installed_version.join('.');
      latest_version = latest_version.join('.');

      if((plugin.installed.version != -1 && latest_version == installed_version) || (cookie_start > -1 && cookie_value == "yes")){
        return true;
      }
      else{
        plugin.valid.error = $.kvm.string.DIALOG_007B;
        plugin.valid.error_msg = $.kvm.string.DIALOG_008A+'<br /><br /> * '+plugin.installed.version+' -> '+$.kvm.plugin.latest + '<br /><br />' + $.kvm.string.DIALOG_008B;
      }
    }
    else{
      plugin.valid.error = $.kvm.string.DIALOG_007A;
      plugin.valid.error_msg = $.kvm.string.DIALOG_008C;
    }
    if(plugin.valid.error){
      dialog([
        '<h3>'+ plugin.valid.error +'</h3>', //STR: Browser plug-ins are not installed.
        '<p>'+ plugin.valid.error_msg +'</p>'
        //STR: Please Install the client software from the <a href="#{href}">download</a> page.
      ].join('\n'), {
        callback: function(button){
          location.href = $('#leftNav li[data-name="downloads"]').length ? '#downloads' : '#settings/downloads';
        },
        cancel_callback: (plugin.valid.error_msg == $.kvm.string.DIALOG_008C) ? null : function(){
          //set cookie
          document.cookie = "plugin_confirm=yes;path=/";
          _callback(true,args);
        },
        labels: {
          ok: $.kvm.string.DIALOG_026A, //STR: download now
          cancel: $.kvm.string.DIALOG_026B //STR: download later
        }
      });
      return false;
    }
  }
  //$.plugin = plugin.valid;

  /** *****************
   * remote access
   ******************** */
  function remoteAccess(force,args) {

    var remote_ok = false;
    if(force == true){
      remote_ok = true;
    }else{
      var propertie =  getProperties(typeof force == 'object' ? force : $(this));
      args = {"addr":propertie.ip, "seq":propertie.seq}
      if(!plugin.valid('remoteAccess',args)) return false;
      else remote_ok = true;
    }
    if(remote_ok == true){
      plugin.installed.RunCommand('rtsp://' + args.addr + '|'+$.kvm.user.name+'|'+$.kvm.user.pw+'|', 'REMOTEKVM');
      setLog('remote', args.seq, '');
      //console.info('remoteAccess rtsp://' + args.addr + '|');
    }else{
      return false;
    }

  }

  /** *************
   * play record
   ************** */
  function playRecord(force,args) {

    var play_ok = false;
    if(force == true){
      play_ok = true;
    }else{
      var propertie =  getProperties($(this));
      args = {"addr":propertie.ip + ":" + propertie.port, "name":propertie.name, "seq":$('#items li.current').attr('id')}
      if(!plugin.valid('playRecord',args)) return false;
      else play_ok = true;
    }
    if(play_ok == true){
      var file_name = args.name.split("_");
      plugin.installed.RunCommand(path = 'http://'+ args.addr + '/files/record/' + file_name[0] + '/' + file_name[1] + '/' + args.name+'.ts|', 'KVMPLAYER');
      setLog('play', args.seq, args.name);
      //console.info('playRecord: ' + 'http://'+ args.addr + '/files/record/' + file_name[0] + '/' + file_name[1] + '/' + args.name+'.ts|');
    }else{
      return false;
    }

  }

  /* form controllers */
  /** ***********************
   * form submit handler
   * @return {Boolean} false
   ************************* */
  function submit(_form) {
    //console.log('submit', this);

    var params
      , restart
      , interval
      , confirmed = !!_form.tagName
      , form = $(confirmed ? _form : this)
      , data = jsonlize(form.serializeArray())
      , changed = compareData(data, update.data) // check for form value changed
      , url = form.attr('action')
      , deviceProc = url.match('devices/')
      , userProc = url.match('users/')
      , settings = url.match('settings/')
      , multipartFormData = form.attr('enctype') == 'multipart/form-data'
      // FIXME: add => create
      , action = url.match(/add|update|delete|index|confirm/)[0];

    //console.log(form.find(':submit').filter(':visible').data('check-change'));
    if(form.find(':submit').filter(':visible').data('check-change') != 'no' && !changed && action != 'delete') {
      status($.kvm.string.STATUS_005, 'warn'); //STR: There are no changes.
      return false;
    }

    // confirm action
    if (deviceProc && action == 'update'
      && ( data.ip4 != update.data.ip4
        || data.subnet4 != update.data.subnet4
        || data.gateway4 != update.data.gateway4
        || data.web_port != update.data.web_port
        || data.dns1 != update.data.dns1
        || data.dns2 != update.data.dns2)) {

      if (!confirmed) {
        dialog([
          '<h3 class="warn">'+ $.kvm.string.DIALOG_009 +'</h3>', //STR: Warning! Restart the device.
          '<p>'+ $.kvm.string.DIALOG_010 +'</p>' //Restart the device after the data synchronized. Are you sure you want to continue?
        ].join('\n'), {
          callback: function() {
            form.append(template([
              '<input name="send_device" value="{seq}" type="hidden" />',
              '<input name="send_user" value="{name}" type="hidden" />'
            ].join('\n'), {
              seq: $.kvm.device.seq,
              name: $.kvm.user.name
            })).attr('action', template('http://{ip4}:{port}/devices/update.php', {
              ip4: data.current_ip4,
              port: data.current_port
            })).attr('method', 'get');

            submit(form.get(0));
            overlay('Updating...');
          }
        });
        return false;
      }
      restart = true;
    }

    if (!confirmed && !this.force && action == 'delete') {
      dialog(template([
        '<h3 class="warn">{title}</h3>',
        '<p>'+ $.kvm.string.DIALOG_011 +'</p>' //STR: Are you sure?
      ].join('\n'), {
        title: form.find('.remove').attr('title')
      }), {
        callback: function() {
          submit(form.get(0));
        }
      });
      return false;
    }

    var form_action_valus = form.find('input[name="action"]').val();
    if (!confirmed && settings && form_action_valus != 'download') {
      dialog([
        '<h3 class="warn">'+ $.kvm.string.DIALOG_012 +'</h3>', //STR: Apply Changes
        '<p>'+ $.kvm.string.DIALOG_013 +'</p>' //STR: Are you sure you want to apply changes?
      ].join('\n'), {
        callback: function() {
          submit(form.get(0));

          if (multipartFormData && form_action_valus == 'upload') {
            dialog.close();
            overlay('Uploading...');
            $message.addClass('loading');
          }
          else if($center.find('.current a').attr('href').match('general')){
            var change_setting = setTimeout(function(){location.reload()},300);
          }
        }
      });
      return false;
    }
    if (multipartFormData) return form.get(0).submit();

    $.ajax({
      url: url,
      type: form.attr('method'),
      data: form.serialize(),
      dataType: restart ? 'jsonp' : url.match(/settings/) ? 'html' : 'json',
      success: function(response) {
        if (action == 'delete') $current.prev().children('a').first().click();

        // 장치 데이터 갱신
        if (deviceProc) {
          if (action == 'update' && response.length && restart) {
            socket(template('http://{ip4}:{port}/devices/reboot.php?device={device}&user={user}', {
              ip4: data.current_ip4,
              port: data.current_port,
              device: $.kvm.device.seq,
              user: $.kvm.user.name
            }));

            interval = setInterval(function() {
              socket(template('http://{ip4}:{port}/ping.php', {
                ip4: data.ip4,
                port: data.web_port
              }), function() {
                clearInterval(interval);
                overlay.close();

                var items = '';
                $.each(response, function() {
                  items += template('<li class="{state}"><strong>{name}</strong> - {state}</li>', $.extend(this, {
                    state: this.state ? 'success' : 'failure'
                  }));
                });
                var dialog_message = $.kvm.string.DIALOG_015.replace('[#number_of_updated_devices#]',response.length);
                dialog([
                  '<h3 class="warn">'+ $.kvm.string.DIALOG_014 +'</h3>', //STR: Completely synchronized //STR: items update following the result:
                  '<p style="padding-bottom: 10px">'+ dialog_message +'</p>',
                  '<ul class="square">' + items + '</ul>'
                ].join('\n'), {
                  "callback": function(button){
                    if($('#items').find('li.local').hasClass('current')) location.href = 'http://'+data.ip4+':'+data.web_port+'/'+location.hash;
                  },
                  "labels": {"ok": $.kvm.string.DIALOG_001A,"cancel": ""}
                });
              });
            }, 3000);
          }

          if (action == 'delete') {
            var spliced = $devices.splice($devices.indexOf(data.id));
            spliced.shift();
            $devices = $devices.concat(spliced);
          }
          if (action == 'add') {
            $devices.push($anchor.attr('href').split('seq=')[1]);
            request($anchor.attr('href'));
          }
          // 디바이스 추가 허용
          if (action == 'confirm') {
            dialog.close();
            request($anchor.attr('href'));
          }
        }

        // 사용자 데이터 갱신
        if (userProc) {
          if (action == 'update') $anchor.html(data.id);
          if (action == 'add'){
            $anchor.html(data.id).attr('href', '/users/?seq=' + response.seq);
            request('http://'+$.kvm.device.ip+":"+$.kvm.device.port+'/users/?seq=' + response.seq);
          }
        }

        $('#center').find('li.current').find('a').click();
        update.data = data;
        //$win.resize();
        status($.kvm.string.STATUS_006, 'info'); //STR: Success!
      }
    });

    return false;
  }

  /** **********************************
   * data comparison
   * @return {Boolean} value changed
   *********************************** */
  function compareData(currentData, workingCopy, reversed) {
    //console.log('compareData');

    var current, compare;
    for (var name in workingCopy) {
      working = workingCopy[name];
      current = currentData[name];
      if (working && !current) return true;
      if (typeof working == 'object' && working.length) {
        if (working.length != current.length) return true;
        for (var i = 0; i < working.length; i++)
          if (working[i] != current[i]) return true;
      } else if (working != current) return true;
    }

    if (!reversed) return compareData(workingCopy, currentData, true);

    return false;
  }

  /** ***************************
   * form activation check
   * @return {Boolean} false
   ***************************** */
  function checkChanged(event) {

    var target = event.target;
    if (event.type == 'keydown') {
      if (target.spotlighted) {
        if (event.keyCode == 9) {
          $('input[name="' + ( target.name == 'ip4' ? 'web_port' : 'ip4') + '"]').focus();
          return false;
        }
        if (event.keyCode == 13) {
          searchDevice.call(target);
          return false;
        }
      }

      if (event.keyCode == 13) {
        $(this).submit();
        return false;
      }
      return;
    }

    //if (update.data.mode && update.data.mode.match(/firmware|database/)) return;

    //console.log('checkChanged', target);


    if(target){
      if (target.getAttribute("id") == "account-type"){
        $(target).change(function(){
          $('#assigned')[this.value == 'user' ? 'show' : 'hide']();
        });
      }
      else if (target.getAttribute("type") == "file"){
        $(target).change(function(){
          uploadFile(target);
        });
      }
    }

    $form.find('.save,.apply,.create').toggleClass('dim', !compareData(jsonlize($form.serializeArray()), update.data))
  }

  /** *******************************************
   * iframe callback for multipart/form-data
   * @param {Object} data object like json
   * @param {Boolean} result status
   ********************************************** */
  win.callback = function(json, result) {
    //console.log('win.callback', arguments);

    if (result) {
      status($.kvm.string.STATUS_007, 'info'); //STR: success
    } else {
      status(json.statusText, 'error');
    }
    overlay.close();
    $message.addClass('loading');
  }

  win.reboot_ping = function(){
    //alert('reboot');
    socket('/devices/reboot.php');

    interval = setInterval(function() {
      socket('/ping.php', function(){
        clearInterval(interval);
        overlay.close();
        location.href = "/signout/";
      });
    },3000);
  }

  /** *********************
   * search device
   ********************** */
  function searchDevice() {
    //console.log('searchDevice');

    var ip, port, input = $(this);
    if (this.name == 'ip4') {
      ip = this.value = this.value.replace(/https?:\/\//, '');

      port = $('input[name="web_port"]');
      if (!port.val()) port.val('80');
      port = port.val();

    } else if(this.name == 'web_port') {
      ip = $('input[name="ip4"]').val();
      port = this.value;
    }

    $form.find('p.error').remove();
    clearTimeout(searchDevice.timeout);
    if (!(ip.match(/\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b/) && port.match(/\b\d{1,4}\b/))) return;

    searchDevice.timeout = setTimeout(function() {
      rightOverlay(true);
      socket('http://' + ip + ':' + port + '/devices/read.php?call_location=external',
        function(data){
          if($devices.indexOf(data[0].seq) != -1) return error($.kvm.string.VALIDATE_022, input); //STR: This device has already been connected
          rightOverlay.close();
          fillData(data[0]);
          $anchor.html(data[0].name);
          //$('input[type!="hidden"]').attr('readonly', true);
          $('.button.add,.button.save,.button.remove').hide();
          $('.create').removeClass('disabled');
          $('.button.cancel,.button.create').show().last().removeClass('dim');
        },
        function(){
          status($.kvm.string.STATUS_008, 'error'); //STR: Reading Device Information Failed!!
          $('[name="subnet4"]').val('');
          $('[name="gateway4"]').val('');
          $('[name="dns1"]').val('');
          $('[name="dns2"]').val('');
          $('[name="seq"]').val('');
          rightOverlay.close();
        }, 2500);
    }, 500);
  }

  /** *****************************
   * getToolsTemplate
   * @param {String} title
   * @return {String} tools html
   ****************************** */
  function getToolsTemplate(title) {
    return template([
      '<div class="tools">',
        //'<a href="#moveUpItem" class="button-move-up" title="Move Up" tabindex="-1">Up</a>',
        //'<a href="#moveDownItem" class="button-move-down" title="Move Down" tabindex="-1">Down</a>',
        '<a href="#{fire}" class="button-del" title="{title}?" tabindex="-1">{title}</a>',
      '</div>'
    ].join('\n'), {
      fire: title.match(/IP/) ? 'removeAddress' : 'removeDevice',
      title: title
    });
  }

  /** *****************************************
   * centerSlide
   * @param {Boolean} show or hide
   * @param {Funciton} after animate callback
   ******************************************** */
  function centerSlide(show, callback){
    //console.log('centerSlide', arguments);
    centerSlide.before = centerSlide.narrow = {left:"219px", width:"550px"};
    centerSlide.after = centerSlide.wide = {left:"-1px", width:"770px"};
    if(show){
      centerSlide.before = centerSlide.wide;
      centerSlide.after = centerSlide.narrow;
    }
    $right.stop().animate(centerSlide.after,440,'customEasing',function(){
      callback && callback();
    });
  }
  /** ***************************
   * add item
   * @return {Boolean} false
   **************************** */
  function addItem() {
    //console.log('addItem', this);
    if ($right.hasClass('wide')){
      centerSlide(true, addItem);
      $right.removeClass("wide").addClass("narrow");
    }
    else {
      var controller = getController()
        , label = getNewLabel()
        , ul = $center.children('ul')
        , li = $('<li class="current"><a href="#new">' + label + '</a></li>')
        , count = ul.find('li').length;

      if (controller == "devices"  && count > 19) {
        status($.kvm.string.STATUS_009, 'error'); //STR: License exceeded
        return false;
      }

      $form.attr('action', '/' + getController() + '/add.php');
      $form.addClass('add-form');
      $('#dns-setting').addClass('last');
      $current = ul.find('li').removeClass('current').last().after(li).next();
      $anchor = $current.children('a').first();
      request($anchor.attr('href'));

      $form.delegate('.goto-device','click',function(){
        return false;
      });
    }
    return false;
  }

  /**
   * save item
   */
  function saveItem() {
    //console.log('saveItem', this);
    $form.attr('action', '/' + getController() + '/update.php');
    $form.submit();
  }

  /**
   * create item
   */
  function createItem(event) {
    //console.log('createItem', this, $form);
    if($(event.target).hasClass('disabled')) return false;
    $form.attr('action', '/' + getController() + '/add.php');
    $form.submit();
  }

  /**
   * remove item
   */
  function removeItem() {
    //console.log('removeItem', this);

    $form.attr('action', '/' + getController() + '/delete.php');
    $form.data('validator', {settings: {rules: {}}});
    $form.submit();
  }

  /**
   * select device
   */
  function selectDevice() {
    //console.log('selectDevice', this);
    var selector = $('#selector').toggle();
    if (selector.is(':visible')) {
      !selector.find('> div > *').show();
      // filtring, selected items
      $('input[name="device_seq[]"]').each(function() {
        $('#seq_' + this.value).hide().next().hide();
      });
      // chaecking, selector have any items
      !selector.find('>div>:visible').length
        && selector.hide()
        && status(selector.attr('title'), 'error');
    }
  }

  /**
   * assign device
   */
  function assignDevice() {
    //console.log('assignDevice', this);

    var parent = $('.accessable-devices')
      , selector = $('#selector')
      , item = $(this).hide().next().hide().end()
      , name = item.next().html()
      , html = template([
        '<div class="item access-device">',
          getToolsTemplate('Remove device'),
          '<label style="width: auto;" class="filename" title="{title}">',
            '<a href="#devices/{name}" class="goto-device">{name}</a>',
          '</label>',
          '<input type="hidden" value="{value}" name="device_seq[]">',
          '<input type="hidden" value="{name}" name="device_name[]">',
        '</div>'
      ].join('\n'), {
        name: name,
        title: item.next().html(),
        value: this.id.replace('seq_', '')
      });

    parent.append(html);
    !item.parent().find('>:visible').length && selector.hide();
  }

  /**
   * remove device
   */
  function removeDevice() {
    //console.log('removeDevice', this);
    var selector = $('#selector')
      , item = $(this).parents('.access-device')
      , seq = item.find('input').val();

    item.remove();
    $('#seq_' + seq).show().next().show();
  }

  /**
   * add allowed ip address
   * @return {Boolean} false
   */
  function addAddress() {
    //console.log('addAddress', this);
    var iptable = $('#iptable');
    iptable.append($(template([
      '<fieldset class="item allowed-ip" style="height: 0;">',
        getToolsTemplate('Delete IP'),
        '<label>IP Address</label> <input id="ip-{index}" name="ip[]" value="" type="text">',
        '<label>Netmask</label> <input id="mask-{index}" name="netmask[]" value="" type="text">',
      '</fieldset>'
    ].join('\n'), {
      index: iptable.find('fieldset').length + 1
    })).animate({height: 84}, 300));

    return false;
  }

  /**
   * moveUpItem
   * @return {Boolean} false
   */
  function moveUpItem() {
    //console.log('moveUpItem', this);
    var item = $(this).parents('.item');
    var prev = item.prev('.item');
    if (prev.length) {
      var height = item.prop('offsetHeight');
      item.css({position: 'relative', backgroundColor: '#ffff9c'}).animate({top: '-=' + height}, 400)
      prev.css('position', 'relative').animate({top: '+=' + height}, 400, function() {
        prev.css('top', 0).before(item.css({top: 0, backgroundColor: ''}))
      });
    }
    return false;
  }

  /**
   * moveDownItem
   * @return {Boolean} false
   */
  function moveDownItem() {
    //console.log('moveDownItem', this);

    var item = $(this).parents('.item');
    var next = item.next('.item');
    if (next.length) {
      var height = item.prop('offsetHeight');
      item.css({position: 'relative', backgroundColor: '#ffff9c', zIndex: 1}).animate({top: '+=' + height}, 400)
      next.css('position', 'relative').animate({top: '-=' + height}, 400, function() {
        item.css({top: 0, backgroundColor: '', zIndex: 0}).before(next.css('top', 0));
      });
    }
    return false;
  }

  /**
   * cancelCreate
   * @return {Boolean} false
   */
  function cancelCreate() {
    //console.log('cancelCreate', this);
    $current.prev().children('a').first().click();
    return false;
  }

  /**
   * remove allowed mac address
   * @return {Boolean} false
   */
  function removeAddress() {
    //console.log('removeAddress', this);

    $(this).parents('.allowed-ip').animate({height: 0}, 300, function() {
      $(this).remove();
    });
    return false
  }

  /**
   * download files
   * @return {Boolean} false
   */
  function downloadFile() {
    //console.log('downloadFile', this);
    // for logging
    var form = $form.append('<input type="hidden" name="action" value="download"/>');
    $form.get(0).submit();
    form.find('input[name=action]').remove();
    $('#software-hover').fadeIn(2000);
    return false;
  }

  /**
   * select file
   */
  function uploadFile(target) {
    //console.log('uploadFile', target);
    //var input = $(this).next().click()
    var input = $(target)
      , value = input.val()
      , btn = input.next();
      //, upload_interval = setInterval(function() {
      if (value != "") {
        //clearInterval(upload_interval);
        $form.find('input[name=action]').remove();
        if ( update.data.mode == 'database' && value.indexOf('.DB3') > 0
          || update.data.mode == 'firmware' && value.indexOf('.zip') > 0){
          $form.append('<input type="hidden" name="action" value="upload" />');
          $('.button.apply').removeClass('dim');
          btn.text(value.substr(value.lastIndexOf("\\")+1)).addClass('picked');
          $form.find('p.error').remove();
        } else {
          input.val('');
          $('.button.apply').addClass('dim');
          btn.text(btn.attr('title')).removeClass('picked');
          error($.kvm.string.ERROR_001, input.parent()); //STR:Invalid file format.
        }
      }
    //}, 1000);
  }

  /**
   * device connect confirm
   */
  function allowConnection(items) {
    //console.log('allowConnection', arguments);

    if (!items.length) items = $current;
    items.each(function() {
      var device = this;
      dialog([
        '<h3>'+ $.kvm.string.DIALOG_016 +'</h3>', //STR: The new device has been detected.
        //STR: From the other device is registered with this device automatically adds the new device. Do you accept the connection of this device?
        '<p>'+ $.kvm.string.DIALOG_017 +'</p>'
      ].join('\n'), {
        callback: function(button) {
          $.ajax({
            url: '/devices/confirm.php',
            type: 'post',
            data: "confirm=true&seq=" + device.id,
            success: function(){
              status($.kvm.string.STATUS_006, 'info'); //STR: Success!
              request('http://'+$.kvm.device.ip+":"+$.kvm.device.port+'/'+getController()+'/?seq='+ device.id);
            }
          });
        },
        labels: {
          ok: $.kvm.string.DIALOG_002A, //STR: accept
          cancel: $.kvm.string.DIALOG_002B //STR: later
        }
      });
    });
  }

  /**
   * update contents
   **/
  $requreDatePicker = true;
  function update(html, href) {

    var this_page = href.replace("http://","").split("/")[1];
    //console.log('update: ', this_page);

    $content.html(html);
    //html = null;
    $right = $('#right');
    $center = $('#center');
    $form = $right.find('form.main');
    $current = $center.find('.current');
    $anchor = $current.children('a').first();
    //location.hash = request.location;
    location.nav = true;

    if (this_page == "screens") {
      $('#screens #items img.current_screen').each(function(index){
        playPreview.error_count[index] = 0;
        if (playPreview.load[index]) {
          clearTimeout(playPreview.load[index]);
          playPreview.load[index] = null;
        }
        var _this = $(this), btn = _this.next(), status = btn.children('img'),
        propertie = getProperties(_this), url = 'http://' + propertie.ip + ':' + propertie.port,
        this_attr = {
          "screen": _this,
          "btn": btn,
          "status": status,
          "url": url
        };
        console.log('play preview', this_attr);
        playPreview(index, this_attr);
      });
    }
    else {
      playPreview.load = [];
      $center.find('.nav').find('a').filter('.name').click(request);
    }

    if(this_page == "records" || this_page == "logs"){
      var head = $("#content-head"),
          body = $("#content-body");
      $requreDatePicker && $.getScript('/public/javascripts/jquery.datepicker.js');
      $("#current-date").val($("#list-table").find("tr").filter(":first").attr("data-date"))
        .mousedown(function(){
          var datefiled = $(this).datepicker({
            onChangeMonthYear: null,
            beforeShow: function(){
              rightOverlay(false);
              var first_date = $("#lastRow").attr('data-first-date').split("-");
              datefiled.datepicker('option','minDate',new Date(first_date[0],first_date[1]-1,first_date[2]))
            },
            dateFormat: 'yy-mm-dd',
            changeYear: true,
            changeMonth: true,
            showMonthAfterYear: true,
            monthNamesShort: ['1','2','3','4','5','6','7','8','9','10','11','12'],
            maxDate: "today",
            onSelect: function(date){
              rightOverlay.overlay.addClass('loading');
              rightOverlay.loading = true;
              $.ajax({
                url: $anchor.attr('href'),
                data: 'date=' + date,
                //dataType: (server_addr == '<?=$_SERVER["HTTP_HOST"]?>') ? 'html' : 'jsonp',
                dataType: 'jsonp',
                success: function(data) {
                  $('#list-table>tbody').html(data);
                  $('#content').scrollTop(0);
                  rightOverlay.loading = false;
                },
                error: function() {
                  //console.error('Doh!');
                  rightOverlay.loading = false;
                }
              });
            },
            onClose: function(){
              if(!rightOverlay.loading) rightOverlay.close();
            }
          });
        });

      setTimeout(function(){
        head.addClass("fixed").css("width",body.width()+"px");
        body.css("padding-top",head.innerHeight()+1+"px");
        $content.scroll(paging);
      },450);
    }else{
      $content.unbind('scroll');
    }

    $devices = [];
    var items = $('#items'), firstTime = !$devices.length;
    items.length && items.find('li').each(function() {
      $devices.push(this.id)
      /*
      if (firstTime){}
      else if (!dialog.onece && $devices.indexOf(this.id) == -1 && $(this).hasClass('new')) {
        // 이벤트성 다이얼로그 호출
        allowConnection(items.find('.new'));
        dialog.onece = true;
      }
      */
    });
    update.data = jsonlize($form.serializeArray());

    var rules = {};
    for (var name in update.data) if ($rules[name]) rules[name] = $rules[name];
    $form.validate({
      rules: rules,
      errorElement: 'p'
    });
  }

  /** *********************
   * log
   ************************ */
  function setLog(action, target, file) {
    request.notrecord = true;
    socket('/logs/add.php?action=' + action + '&target=' + target + '&file=' + file);
  }

  /** **********************************
   * socket
   * @param {String} url
   * @param {Object} success callback
   * @param {Object} error callback
   ************************************ */
  function socket(url, success, error, timeout) {
    request.ajax = $.ajax({
      dataType: 'jsonp',
      timeout: timeout ? timeout : '6000',
      url: url,
      beforeSend: function() {
        if (request.ajax && request.ajax.readyState != 4) request.ajax.abort();
        if (this.dataType == 'jsonp') //console.info(this.url);
        /*
        for (var i = 0; i < request.errors.length; i++) {
          if (this.url.match(request.errors[i])) return true;
        }
        */
        clearTimeout(request.timeout);
        if (status.refresh) status();
        $message.addClass('loading');
      },
      success: success || function(data) {
        //console.log(this, data);
      },
      error: error || function(xhr) {
        if (xhr.statusText == 'abort' && this.jsonpCallback) {
          win[this.jsonpCallback] = function() {
            //console.warn('aborted data', xhr);
            delete win[this.jsonpCallback];
          }
        }
        if (xhr.status == 500){
          var data = $.parseJSON(xhr.responseText);
          if (data && typeof data == 'object') status(data.statusText, data.statusType);
        }
      }
    });
  }

  /** *************************
   * request contents
   * @return {Boolean} false
   **************************** */
  request.errors = [];
  function request(event, page, callback) {
    //console.log('request', arguments);
    //cahngeLocation && clearInterval(cahngeLocation);
    location.nav = false;
    var currentItem = $(this)
      .parents('ul')
        .find('li.current')
          .removeClass('current')
        .end()
      .end()
    .parent()
      .addClass('current');

    //$('.overlay').remove();
    var href = typeof event == 'string' ? event : this.href;
    if(!href){
      //console.error('request: No URL');
      return false;
    }

    if (href.match('#new')) {
      emptyData();
      update.data = jsonlize($form.serializeArray());
      $('.button.add,.button.save,.button.remove').hide();
      $('.button.create,.button.cancel').show();
      $('.button.create').toggleClass('dim', true);
      return false;
    }

    if (href.match('#')) return //console.warn('WTF?');

    var currentController = getController()
      , isData = href.match('list.php')
      , isNew = currentItem.hasClass('new');

    if (!page && !request.model) {
      var model = !currentItem.parents('#left').length && href.match(currentController) && $('#center li.current').not(':first-child').attr('data-name');
      request.location = '#' + getController(href) + (model ? '/' + model : '');
      location.hash = request.location;
    }

    moreAction.close();
    dialog.close();

    if (isNew) {
      allowConnection(currentItem);
      dialog.onece = true;
    }

    request.ajax = $.ajax({
      url: href,
      dataType: isData && (!href.match(doc.domain)) && !href.match('devices') ? 'jsonp' : 'html',

      beforeSend: function() {
        if (request.ajax && request.ajax.readyState != 4) request.ajax.abort();
        if (this.dataType == 'jsonp') //console.info(this.url);
        clearTimeout(request.timeout);
        if (status.refresh) status();
        $message.addClass('loading');
        if(page){
          current_page = href.match(/logs|records/) && $('#page-loading').show()
        }else{
          current_page = href.match(/logs|records/) && rightOverlay(true);
          $('#record-detail').appendTo("#content-body").hide().find(".snapshots img").remove();
        }
      },

      success: function(data) {
        !page && $content.scrollTop(0);
        if(typeof data == 'object') return status($.kvm.string.STATUS_010, 'error'); //STR: Data Type Error
        paging.noMore = page && !data;
        if ((current_page = href.match(/logs|records/)) && isData) {
          $("#lastRow").remove();
          $('#list-table>tbody')[page ? 'append' : 'html'](data);
          $current = $center.find('.current');
          $anchor = $current.children('a').first();
          callback && callback();
          if(page){
            $('#page-loading').hide();
          }
          else{
            $("#current-date").val($("#list-table").find("tr").filter(":first").attr("data-date"));
          }

        } else {
          var wasVisible = $right ? $right.hasClass('narrow') : false, isVisible = data.indexOf('id="right" class="narrow') != -1;
          (isVisible == true) ? $("#kvmWrap").addClass("triple") : $("#kvmWrap").removeClass("triple");
          if(wasVisible == isVisible){
            update(data, href);
          }else{
            isVisible && update(data.replace('id="right"','id="right" style="width:770px; left:-1px;"'), href);
            centerSlide(isVisible, function(){ wasVisible && update(data, href); });
          }
        }
        if(request.model != null){
          location.nav = false;
          var sub_location = request.model;
          request.model = null;
          setTimeout(function(){
            $('#center li[data-name="'+ sub_location +'"]>a').click();
            sub_location = null;
          },550);
        }

      }
    });
    return false;
  }

  /** ******************************
   * setup current navigation item
   ******************************* */
  function nav() {
    var controller = getController();
    var model = location.hash.split('/')[1];
    if (model) request.model = model;
    if (this.tagName) {
      $(this).parent()[this.href.match(controller) ? 'addClass' : 'removeClass']('current');
    } else {
      $('#leftNav li[data-name="'+ controller.toLowerCase() +'"]>a').click();
    }
  }

  /** ************************************
   * paging event
   * @param {Object} event from scroll
   ************************************* */
  function paging(event) {
    var el = this,
        head = $("#content-head"), head_height = head.innerHeight(),
        body = $("#content-body"), $date;
    head.addClass("fixed").css("width",body.width()+"px");
    body.css("padding-top",head_height+"px");

    var position = Math.abs(body.offset().top) + head_height;
    $.each($("#list-table>tbody>tr"),function(){
      if($(this).position().top >= position){
        $date = $(this).prev().is("tr") ? $(this).prev().attr("data-date") : $(this).attr("data-date");
        return false;
      }
    });
    $("#current-date").blur().val($date).datepicker("hide");

    var lastRow = $("#lastRow");
    paging.timer && clearTimeout(paging.timer);
    paging.timer = setTimeout(function () {
      var id = $("#lastRow").attr('data-last-index');
      if (id && el.scrollTop > el.scrollHeight - el.offsetHeight - 150) {
        if (paging.noMore) status($.kvm.string.STATUS_011, 'warn'); //STR: No more data.
        else request($anchor.attr('href') + '&start=' + id, true);
      }
    }, 200);
    //$win.resize();
  }

  function moreRecords() {
    var target = $(this);
    request(target.attr("data-href"), true, function() {
      target.parent().remove();
    });
    return false;
  }

  /** *******************
   * play preview
   ******************** */
  function playPreview(index,args){
    //console.log('playPreview', this);
    if(getController() != "screens") return false;
    request.ajax = $.ajax({
      url: args.url+'/screens/read.php',
      dataType: 'jsonp',
      timeout: 2000,
      beforeSend: function(){
        playPreview.load[index] && clearTimeout(playPreview.load[index]);
      },
      success: function(data){
        var status = (typeof data == 'string') ? data : data.status;
        args.btn.attr("class","screenOverlay "+status);
        if(status == 'normal') args.screen.attr("src",args.url+"/screens/screen.php#"+ new Date().getTime());
        //if(data == 'normal') args.screen.attr("src",args.url+"/files/snapshot/screen.jpg#"+ new Date().getTime());
        playPreview.error_count[index] = 0;
      },
      error: function(xhr){
       //console.log(playPreview.error_count[index]);
       if(playPreview.error_count[index] > 2){
         args.btn.attr("class","screenOverlay notconnect");
         args.screen.attr("src","/public/images/blank.gif");
       }else{
         playPreview.error_count[index]++;
       }
      },
      complete: function(){
        playPreview.load[index] && clearTimeout(playPreview.load[index]);
        playPreview.load[index] = setTimeout(function() {
          playPreview(index,args);
        }, 8000);
      }
    });
  }
  playPreview.load = [];
  playPreview.error_count = [];

  /** *****************
   * showDescription
   ****************** */
  Number.prototype.to2 = function(){ return (this > 9 ? "" : "0") + this; };
  Date.MONTHS = [$.kvm.string.DATE_JAN, $.kvm.string.DATE_FEB, $.kvm.string.DATE_MAR, $.kvm.string.DATE_APR, $.kvm.string.DATE_MAY, $.kvm.string.DATE_JUN,
                 $.kvm.string.DATE_JUL, $.kvm.string.DATE_AUG, $.kvm.string.DATE_SEP, $.kvm.string.DATE_OCT, $.kvm.string.DATE_NOV, $.kvm.string.DATE_DEC];
  Date.DAYS = [$.kvm.string.DATE_SUN, $.kvm.string.DATE_MON, $.kvm.string.DATE_TUE, $.kvm.string.DATE_WED,
               $.kvm.string.DATE_TUR, $.kvm.string.DATE_FRI, $.kvm.string.DATE_SAT];
  Date.AMPM = [$.kvm.string.DATE_AM, $.kvm.string.DATE_PM];
  Date.prototype.getDateString = function(dateFormat){
      var result = "";
      dateFormat = dateFormat == 8 && "YYYY.MM.DD" || dateFormat == 6 && "hh:mm:ss" || dateFormat || "YYYY.MM.DD hh:mm:ss";
      for (var i = 0; i < dateFormat.length; i++) {
        result +=
          dateFormat.indexOf("YYYY", i) == i ? (i += 3, this.getFullYear()) :
          dateFormat.indexOf("YY", i) == i ? (i += 1, String(this.getFullYear()).substring(2)) :
          dateFormat.indexOf("MMM", i) == i ? (i += 2, Date.MONTHS[this.getMonth()]) :
          dateFormat.indexOf("MM", i) == i ? (i += 1, (this.getMonth() + 1).to2()) :
          dateFormat.indexOf("M", i) == i ? (this.getMonth() + 1) :
          dateFormat.indexOf("DDD", i) == i ? (i += 2, Date.DAYS[this.getDay()]) :
          dateFormat.indexOf("DD", i) == i ? (i += 1, this.getDate().to2()) :
          dateFormat.indexOf("D", i) == i ? (this.getDate()) :
          dateFormat.indexOf("a", i) == i ? (this.getHours() > 11 ? Date.AMPM[1]:Date.AMPM[0]) :
          dateFormat.indexOf("hh", i) == i ? (i += 1, this.getHours().to2()) :
          dateFormat.indexOf("h", i) == i ? (this.getHours()) :
          dateFormat.indexOf("mm", i) == i ? (i += 1, this.getMinutes().to2()) :
          dateFormat.indexOf("m", i) == i ? (this.getMinutes()) :
          dateFormat.indexOf("ss", i) == i ? (i += 1, this.getSeconds().to2()) :
          dateFormat.indexOf("s", i) == i ? (this.getSeconds()) : (dateFormat.charAt(i));
      }
      return result;
  };

  function showDescription(event) {
    //console.warn('showDescription', this);
    var trigger = this, $trigger = $(this), target = $trigger.next();
    var currnt_top = $content.scrollTop();
    target.toggle(0, function(){
      var is_visible = target.is(':visible');
      $trigger.toggleClass('opened', is_visible);
      target.toggleClass('opened', is_visible);
      if (getController() == 'records') {
        var record_detail = $("#record-detail");
        if(showDescription.current){
          $current = $(showDescription.current);
          showDescription.current != trigger && $current.removeClass('opened').next().hide();
          record_detail.find(".snapshots img").appendTo($current.next());
        }
        if(is_visible){
          var snapshots = record_detail.find(".snapshots");
          target.find("img").prependTo(snapshots);

          var first_img = snapshots.find("img:first");
          first_img.attr("data-src") && first_img.attr("src",first_img.attr("data-src")).removeAttr('data-src');
          record_detail.show().appendTo($("td",target));
          snapshots.mousemove(function(event){
            var images = snapshots.find('img').hide()
              , scope = 230 / (images.length - 1)
              , left = event.clientX - snapshots.offset().left - 3
              , index = Math.round(left / scope);
            if (images.length <= index) index = images.length - 1;
            var now = images.eq(index), dataSrc = now.attr('data-src');
            if (dataSrc) now.attr('src', dataSrc).removeAttr('data-src');
            now.show();
            snapshots.find('.time').text(now.attr('alt'));
          });

          var time_arr = $trigger.find(".time").text().split(" ~ "),
              start_time_arr = time_arr[0].split(/[-:\s]/),
              end_time_arr = time_arr[1].split(/[-:\s]/),
              start_time = new Date(start_time_arr[0],start_time_arr[1]-1,start_time_arr[2],start_time_arr[3],start_time_arr[4],start_time_arr[5]),
              end_time = new Date(end_time_arr[0],end_time_arr[1]-1,end_time_arr[2],end_time_arr[3],end_time_arr[4],end_time_arr[5]);
          time_arr = start_time_arr = end_time_arr = null;

          var state_text = $trigger.hasClass("remote") ?
              $.kvm.string.STATE_REMOTE + " (" + $.kvm.string.REMOTE_USER + ": "+ $trigger.attr('data-user')+ ")" :
              $.kvm.string.STATE_IDLE;
          record_detail.find('h5 a').text(target.attr('data-file-name'));
          record_detail.find('.start').text(start_time.getDateString($.kvm.string.DATE_FORMAT));
          record_detail.find('.end').text(end_time.getDateString($.kvm.string.DATE_FORMAT));
          record_detail.find('.length').text($trigger.find('.length').text().replace("h",$.kvm.string.TIME_HOUR).replace("m",$.kvm.string.TIME_MIN).replace("s",$.kvm.string.TIME_SEC));
          record_detail.find('.resolution').text($trigger.find('.resolution').text().replace('*',' '));
          record_detail.find('.size').text($trigger.find('.size').text());
          record_detail.find('.state').text(state_text);
        }else{
          record_detail.appendTo("#content-body").hide();
        }
      }
    });

    showDescription.current = this;
    $content.scrollTop(currnt_top);
  }

  function rebootDevice(){
    //console.log('rebootDevice', arguments);
    var address = $.kvm.device.ip + ":" + $.kvm.device.port;
    if(moreAction.anchor != null){
      var propertie = getProperties();
      address = propertie.ip + ":" + propertie.port;
    }
    //STR: Reboot Device/Are you sure to reboot device?
    dialog('<h3 class="warn">'+ $.kvm.string.DIALOG_018 +'</h3> <p>'+ $.kvm.string.DIALOG_019 +'</p>',{
      callback: function(){
        socket('http://'+address+'/devices/reboot.php?device='+$.kvm.device.seq+'&user='+$.kvm.user.seq);
        if(address == $.kvm.device.ip + ":" + $.kvm.device.port){
          overlay('Rebooting...');
          interval = setInterval(function() {
            socket('/ping.php', function(){
              clearInterval(interval);
              overlay.close();
              location.href = "/signout/";
            });
          },3000);
        }
      }
    });
  }

  function wakepc(){
    var address = $.kvm.device.ip + ":" + $.kvm.device.port;
    if(moreAction.anchor != null){
      var propertie = getProperties();
      address = propertie.ip + ":" + propertie.port;
    }
    $.ajax({
      url: 'http://'+address+'/_common/wol.php?device='+$.kvm.device.seq+'&lang='+$.kvm.language,
      type: 'get',
      dataType: 'jsonp',
      timeout: 3000,
      beforeSend: function(){},
      success: function(data){
        status(data.statusText, data.statusType);
        //console.log('success',data);
      },
      error: function(xhr){
        status($.kvm.string.STATUS_014, 'error');
      },
      complete: function(xhr){}
    });
  }

  function initDatabase(){
    //STR: Initialization Database/Are you sure Initialization Database?

    dialog('<h3 class="warn">'+ $.kvm.string.DIALOG_020 +'</h3> <p>'+ $.kvm.string.DIALOG_021 +
           '</p><p class="ipt"><label>'+ $.kvm.string.STR_SUPER_PW +'</label><br><input type="password" id="dialog_pwd" name="dialog_pwd" /></p>',{
      callback: function(){
        overlay('Initializing...');
        $.ajax({
          url: '/_common/db_init.php',
          type: 'post',
          data: "password=" + encodeURIComponent($("#dialog_pwd").val()),
          success: function(){
            location.href = "/setup/";
          },
          error: function(xhr){
            var data = $.parseJSON(xhr.responseText);
            status(data.statusText, data.statusType);
            overlay.close();
          }
        });
      },
      cancel_callback: function(){
        return false;
      }
    });
  }

  function emptyHDD(){
    //STR: Empty Disk/Are you sure delete all records?
    dialog('<h3 class="warn">'+ $.kvm.string.DIALOG_022 +'</h3> <p>'+ $.kvm.string.DIALOG_023 +
           '</p><p class="ipt"><label>'+ $.kvm.string.STR_SUPER_PW +'</label><br><input type="password" id="dialog_pwd" name="dialog_pwd" /></p>',{
      callback: function(){
        overlay('Deleting...');
        $.ajax({
          url: '',
          type: 'post',
          data: "password=" + encodeURIComponent($("#dialog_pwd").val()),
          success: function(){
            overlay.close();
          },
          error: function(xhr){
            var data = $.parseJSON(xhr.responseText);
            status(data.statusText, data.statusType);
          }
        });
      },
      cancel_callback: function(){
        return false;
      }
    });
  }

  function factoryReset(){
    //STR: Factory Reset/Are you sure reset device to factory setting?
    dialog('<h3 class="warn">'+ $.kvm.string.DIALOG_024 +'</h3><p>'+ $.kvm.string.DIALOG_025 +'</p>',{
      callback: function(){
        alert("ok");
      },
      cancel_callback: function(){
        return false;
      }
    });
  }

  function rview(command){
    if(!command) return false;
    var rv_dialog_forms = $('#rview-dialog-forms')
      , dialog_body
      , request_url
      , launchWin
      , dialogs = {
        //setup
        'setupUsercheck':{
          'win': $('#rview-setup-usercheck'),
          'submit': function(){
            if(postDataGetResponse('userCheck')){
              dialogs.setupUsercheck.win.appendTo(rv_dialog_forms).find('input.text').val('');
              dialog_body.append(dialogs.setupAgent.win);
            }else{
              return false;
            }
          }
        },

        'setupAgent': {
          'win': $('#rview-setup-agent'),
          'submit': function(){
            if(postDataGetResponse('regAgent')){
              dialogs.setupAgent.win.appendTo(rv_dialog_forms).find('input.text').val('');
              dialog_body.append(dialogs.setupComplete.win);
            }
          }
        },

        'setupComplete': {
          'win': $('#rview-setup-complete'),
          'submit': function(){
            dialog.close();
            $('#center').find('li.current').find('a').click();
          }
        },

        //delete
        'deleteAgent': {
          'win': $('#rview-delete-agent'),
          'submit': function(){
            if(postDataGetResponse('delete')){
              dialogs.deleteAgent.win.appendTo(rv_dialog_forms).find('input.text').val('');
              dialog_body.append(dialogs.deleteComplete.win);
            }
          }
        },
        'deleteComplete': {
          'win': $('#rview-delete-complete'),
          'submit': function(){
            dialog.close();
            $('#center').find('li.current').find('a').click();
          }
        },

        //account change
        'accountChange': {
          'win': $('#rview-account-change'),
          'submit': function(){
            if(postDataGetResponse('change')){
              dialogs.accountChange.win.appendTo(rv_dialog_forms).find('input.text').val('');
              dialog_body.append(dialogs.accountChangeComplete.win);
            }
          }
        },
        'accountChangeComplete': {
          'win': $('#rview-account-change-complete'),
          'submit': function(){
            dialog.close();
            $('#center').find('li.current').find('a').click();
          }
        }
      }
    , action = {
      'setup':{
        url: '/_rview/agent_setup.php',
        dialog: dialogs.setupUsercheck.win,
        callback: function(){
          var server_addr = $('input[data-name=rvdata-SERVERADDR]').prop('disabled',true).val('https://biz.rview.com').addClass('disabled');
          $('input[name="rviewAuthType"]').unbind('click').click(function(){
            server_addr.prop('disabled',!!(this.value=='biz')).val(this.value=='biz' ? 'https://biz.rview.com':'https://')[this.value=='biz' ? 'addClass':'removeClass']('disabled');
            !!(this.value == 'asp') && server_addr.focus();
          }).eq(0).prop('checked',true)

          //$('input[data-name=rvdata-SERVERADDR]').val('https://bbiz.rview.com');
          //$('input[data-name=rvdata-DOMAIN]').val('remotekvm');
          //$('input[data-name=rvdata-WEBID]').val('admin');
          //$('input[data-name=rvdata-WEBPASS]').val('dkf4430');
        }
      },
      'delete':{
        url: '/_rview/agent_delete.php',
        dialog: dialogs.deleteAgent.win
      },
      'accountchange':{
        url: '/_rview/agent_account_change.php',
        dialog: dialogs.accountChange.win
      }
    }[command];

    dialog(action.dialog.html());
    dialog_body = $('#dialog').find('.dialog-body').html('').append(action.dialog);
    dialog_body.find('input.text').val('');
    action.callback && action.callback.call();


    $('.btn-rview-cancel').unbind('click').click(dialog.close);
    $('.btn-rview-next').unbind('click').click(function(){
      dialogs[$(this).parents('.rview-dialog').attr('data-step')].submit();
    });

    //utils -----------------------------------------------
    function postDataGetResponse(step){
      if(!step) return false;

      var params = [];
      dialog_body.find(':text,:password,:hidden').filter(':not(:disabled)').each(function(){
        var ipt = $(this);
        ipt.val(ipt.val().replace(/[\s]+/,''));
        params.push(ipt.attr('data-name').replace('rvdata-','')+'='+ipt.val());
      });

      var result;
      $.ajax({
        url: action.url,
        type: 'post',
        async: false,
        data: 'step='+step+'&'+params.join('&'),
        dataType: 'json',
        timeout: 6000,
        beforeSend: null,
        success: function(data){
          if(data == null){
            alert($.kvm.string.ALERT_001+'/null');
            result = false;
          }
          if(data.type=='error'){
            alert(data.message.replace(/\/n\//g,'\n'));
            result = false;
          }
          if(data.type=='ok'){
            for(var key in data)
              key != 'type' && key != 'ERRORMSG' && key != 'RESULT' && key != 'post_fileds' &&
                $('<input type="hidden" />').attr('name','rvdata-'+key).attr('data-name','rvdata-'+key).val(data[key]).appendTo(dialog_body);
            result = true;
          }
        },
        error: function(){
          alert($.kvm.string.ALERT_001); //error
          result = false;
        },
        complete: null
      });

      return result;
    }
  }

  function rviewSetup(){
    rview('setup');
  }

  function rviewDelete(){
    rview('delete');
  }

  function rviewAccountChange(){
    rview('accountchange');
  }

  function startRecord(){
    alert('start record');
  }

  function stopRecord(){
    alert('stop record');
  }

  /* fire event
  fire.actions = {
    screenAction: screenAction,
    moreAction: moreAction,
    remoteAccess: remoteAccess,
    playRecord: playRecord,
    downloadRecord: downloadRecord,
    deleteRecord: deleteRecord,
    moreRecords: moreRecords,
    addItem: addItem,
    saveItem: saveItem,
    removeItem: removeItem,
    createItem: createItem,
    cancelCreate: cancelCreate,
    selectDevice: selectDevice,
    assignDevice: assignDevice,
    removeDevice: removeDevice,
    addAddress: addAddress,
    removeAddress: removeAddress,
    moveDownItem: moveDownItem,
    moveUpItem: moveUpItem,
    downloadFile: downloadFile,
    allowConnection: allowConnection,
    rebootDevice: rebootDevice,
    initDatabase: initDatabase,
    emptyHDD: emptyHDD,
    factoryReset: factoryReset,
    rviewAuth: rviewAuth
  };

  function fire(event) {
    //console.info('fire', event);
    moreAction.close();

    var target = event.target,
        _target = $(target),
        command;
    if(target){
        if (target.tagName != 'A') target = $(target).parents('a').get(0);
        command = target && target.href && target.href.split('#')[1];
    }

    var run_function = fire.actions[command];
    if (run_function) {
      run_function.call(target, event);
      return false;
    }

    //  else if (command == "settings/Firmware" && _target.parent('#message').length) {
    //    status();
    //    checkUpdate.confirmed = true;
    //  }

  }
  // */


  /** *************************
   * coockies
   ************************** */
  function getCookie(name){
    var cookies = doc.cookie.replace(/\s/g,'').split(';')
      , result = {} , set;
    for(var i in cookies){
      set = cookies[i].split('=');
      result[set[0]] = set[1];
    }
    if(name){
      if(result[name] != undefined) return result[name];
      else return null;
    }
    else{
      if(result.length) return result;
      else return null;
    }
  }

  function setCookie(name,value,expires,path){
    if(!name) return false;
    !value && (value = '');
    var cookie = name +'='+ encodeURIComponent(value) +';path=/'+ (path ? path:'');
    if(expires){
      var date = new Date();
      date.setDate(date.getDate() + expires);
      cookie += ';expires=' + date.toGMTString() + ';';
    }
    doc.cookie = cookie;
  }

  function delCookie(name){
    return setCookie(name,'',-1);
  }

  /** ************
   * initialize
   ************* */
  $(function(){
    $win.resize(function(){
      if(dialog.frame && dialog.frame.length){
        var margin_top = ($win.height() - dialog.frame.height()) / 2 - 50;
        dialog.frame.css('margin-top', margin_top > 15 ? margin_top : 15);
      }
    });

    $("#kvmWrap")
      .click(moreAction.close)
      .delegate('tr.summary','click',showDescription)
      .delegate('form','submit',submit)
      .delegate('input,select,textarea','keydown keyup mouseup mouseenter',checkChanged)
      .delegate('a','click',function(event){
        var target = this
          , func = this.href.split('#');
        //console.log('f',func)
        if($.isArray(func)){
          func = func[1];
          if(func == 'new'){
            return false;
          }else if(func.match($controllers)){
            return;
          }else if(typeof eval(func) == 'function'){
            event.preventDefault();
            eval(func).call(target, event);
            return false;
          }
        }
      })
      .delegate('#sorting-options button.active','click',function(){
        var $this = $(this)
          , $other = $this.siblings()
          , cookie_name = $this.parent().attr('data-cookieName')
          , sortby = $this.attr('id').replace('sort-by-','');
        //console.log($this,$other,$this.parent(),$this.parent().attr('data-cookieName'));
        $this.removeClass('active');
        $other.addClass('active');
        setCookie(cookie_name,sortby);

        var $list = $this.parent().next()
          , $item = $list.children('li')
          , order_list = []
          , key_list = []
          , order_key = sortby == 'name' ? 'data-name' : 'data-order';
        $item.each(function(index){
          var key = $(this).attr(order_key);
          order_list[key] = $(this);
          key_list.push(key);
        });
        key_list.sort();

        $list.empty();
        for(var i in key_list){ $list.append(order_list[key_list[i]]); }
        $list.find('a').filter('.name').click(request);
      });

    $(doc).bind('keypress',function(event){
      if(dialog.visible){
        if(event.keyCode == '13'){
          event.preventDefault();
          $(event.target).is('input') && dialog.frame.find('button.blue').eq(0).click();
        }
        if(event.keyCode == '27'){
         dialog.close();
        }
      }
    });

    $('#leftNav').find('li').find('a').each(nav).click(request).parent('.current').find('a').click();

    location.nav = false;
    var cahngeLocation = setInterval(function(){
      if (location.nav && request.location != doc.location.hash) nav();
    }, 100);
  });

/*
  win.onhashchange = function(){
    nav();
  }
*/
})(jQuery, this, document);
